#
# Cookbook:: dhcp
# Resource:: service
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

include Dhcp::Cookbook::Helpers

property :ip_version, [Symbol, String],
          equal_to: [:ipv4, :ipv6, 'ipv4', 'ipv6'],
          coerce: proc { |p| p.is_a?(Symbol) ? p : p.to_sym },
          default: :ipv4,
          description: 'The IP version, 4 or 6'

property :service_name, String,
          coerce: proc { |p| "#{p}.service" },
          default: lazy { dhcpd_service_name(ip_version) },
          description: 'Override the default service names'

property :systemd_unit_content, [String, Hash],
          default: lazy { dhcpd_systemd_unit_content(ip_version) },
          description: 'Override the systemd unit file contents'

property :config_file, String,
          default: lazy { dhcpd_config_file(ip_version) },
          description: 'The full path to the DHCP server configuration on disk'

property :config_test, [true, false],
          default: true,
          description: 'Perform configuration file test before performing service action'

action_class do
  def do_service_action(resource_action)
    with_run_context(:root) do
      edit_resource(:execute, "Run pre service #{resource_action} #{dhcpd_service_name(new_resource.ip_version)} configuration test.") do
        command dhcpd_config_test_command(new_resource.ip_version, new_resource.config_file)
        only_if { new_resource.config_test && %i(start restart reload).include?(resource_action) && ::File.exist?(new_resource.config_file) }

        action :nothing
      end

      edit_resource(:service, new_resource.service_name.delete_suffix('.service')) do
        notifies :run, "execute[Run pre service #{resource_action} #{dhcpd_service_name(new_resource.ip_version)} configuration test.]", :before

        action :nothing
        delayed_action resource_action
      end
    end
  end
end

action :create do
  with_run_context :root do
    edit_resource(:systemd_unit, new_resource.service_name) do |new_resource|
      content new_resource.dhcpd_systemd_unit_content(new_resource.ip_version, new_resource.config_file)
      triggers_reload true
      verify false

      action :create
    end if dhcpd_use_systemd?
  end
end

action :delete do
  do_service_action([:stop, :disable])
  with_run_context :root do
    edit_resource(:systemd_unit, new_resource.service_name).action(:delete) if dhcpd_use_systemd?
  end
end

action :start do
  do_service_action(action)
end

action :stop do
  do_service_action(action)
end

action :restart do
  do_service_action(action)
end

action :reload do
  do_service_action(action)
end

action :enable do
  do_service_action(action)
end

action :disable do
  do_service_action(action)
end
